

# def rename_files(orig='(.*)[.]asc', new='\1.adoc'):
#     pass
# git mv 'Appendix A -- Your NLP Tools.asc' 'Appendix A -- Your NLP Tools.adoc'
# git mv 'Appendix B -- Playful Python and Regular Expressions.asc' 'Appendix B -- Playful Python and Regular Expressions.adoc'
# git mv 'Appendix C -- Vectors and Matrices (Basic Linear Algebra).asc' 'Appendix C -- Vectors and Matrices (Basic Linear Algebra).adoc'
# git mv 'Appendix D -- Machine Learning.asc' 'Appendix D -- Machine Learning.adoc'
# git mv 'Appendix E -- AWS GPU.asc' 'Appendix E -- AWS GPU.adoc'
# git mv 'Appendix F -- Locality Sensitive Hashing.asc' 'Appendix F -- Locality Sensitive Hashing.adoc'
# git mv 'Chapter 00 -- About this Book.asc' 'Chapter 00 -- About this Book.adoc'
mv 'Chapter 00 -- Acknowledgments.asc' 'Chapter 00 -- Acknowledgments.adoc'
# git mv 'Chapter 00 -- Foreword by Arwen Griffioen.asc' 'Chapter 00 -- Foreword by Arwen Griffioen.adoc'
# git mv 'Chapter 00 -- Preface.asc' 'Chapter 00 -- Preface.adoc'
# git mv 'Chapter 01 - 04 -- Part 01 -- Wordy Machines (Vector Models of Natural Language).asc' 'Chapter 01 - 04 -- Part 01 -- Wordy Machines (Vector Models of Natural Language).adoc'
# git mv 'Chapter 01 -- Packets of Thought (NLP Overview).asc' 'Chapter 01 -- Packets of Thought (NLP Overview).adoc'
mv 'Chapter 02 -- Build Your Vocabulary (Word Tokenization).asc' 'Chapter 02 -- Build Your Vocabulary (Word Tokenization).adoc'
# git mv 'Chapter 03 -- Math with Words (TF-IDF Vectors).asc' 'Chapter 03 -- Math with Words (TF-IDF Vectors).adoc'
# git mv 'Chapter 04 -- Finding Meaning in Word Counts (Semantic Analysis).asc' 'Chapter 04 -- Finding Meaning in Word Counts (Semantic Analysis).adoc'
mv 'Chapter 05 - 10 -- Part 02 -- Deeper Learning (Neural Networks).asc' 'Chapter 05 - 10 -- Part 02 -- Deeper Learning (Neural Networks).adoc'
git mv 'Chapter 05 -- Baby Steps with Neural Networks (Perceptrons and Backpropagation).asc' 'Chapter 05 -- Baby Steps with Neural Networks (Perceptrons and Backpropagation).adoc'
git mv 'Chapter 06 -- Reasoning with Word Vectors (Word2vec).asc' 'Chapter 06 -- Reasoning with Word Vectors (Word2vec).adoc'
mv 'Chapter 07 -- Getting Words in Order with Convolutional Neural Networks (CNNs).asc' 'Chapter 07 -- Getting Words in Order with Convolutional Neural Networks (CNNs).adoc'
git mv 'Chapter 08 -- Loopy (Recurrent) Neural Networks (RNNs).asc' 'Chapter 08 -- Loopy (Recurrent) Neural Networks (RNNs).adoc'
git mv 'Chapter 09 -- Improving Retention with Long Short-Term Memory Networks (LSTMs).asc' 'Chapter 09 -- Improving Retention with Long Short-Term Memory Networks (LSTMs).adoc'
git mv 'Chapter 10 -- Sequence to Sequence Models and Attention (Generative Models).asc' 'Chapter 10 -- Sequence to Sequence Models and Attention (Generative Models).adoc'
git mv 'Chapter 11 - 15 -- Part 03 -- Getting Real (Real World NLP Challenges).asc' 'Chapter 11 - 15 -- Part 03 -- Getting Real (Real World NLP Challenges).adoc'
git mv 'Chapter 11 -- Information Extraction (Named Entity Extraction and Question Answering).asc' 'Chapter 11 -- Information Extraction (Named Entity Extraction and Question Answering).adoc'
git mv 'Chapter 12 -- Stackable Deep Learning (Attention, Networks, Transformers, BERT, GPT).asc' 'Chapter 12 -- Stackable Deep Learning (Attention, Networks, Transformers, BERT, GPT).adoc'
git mv 'Chapter 13 -- Getting Creative (Summarization, Paraphrasing, Generation).asc' 'Chapter 13 -- Getting Creative (Summarization, Paraphrasing, Generation).adoc'
git mv 'Chapter 14 -- Scaling Up (Optimization, Parallelization and Batch Processing).asc' 'Chapter 14 -- Scaling Up (Optimization, Parallelization and Batch Processing).adoc'
git mv 'Chapter 15 -- Getting Chatty (Dialog Engines).asc' 'Chapter 15 -- Getting Chatty (Dialog Engines).adoc'
git mv 'Glossary.asc' 'Glossary.adoc'
git mv 'MEAP--Welcome.asc' 'MEAP--Welcome.adoc'
git mv 'MQR.asc' 'MQR.adoc'
git mv 'Resources.asc' 'Resources.adoc'
git mv 'TOC.asc' 'TOC.adoc'
